#!/usr/bin/python

# Random_Circles is a program for randomly generating epicyclic orbits.
# Epicycles are an old (and wrong) model of orbital mechanics, but they do make cool pictures.
# Copyright Sam Kennerly 2010 (Creative Commons Attribution/Noncommercial/Share-Alike 3.0)
# http://creativecommons.org/licenses/by-nc-sa/3.0/

# To run this program, type python Random_Circles.py in the terminal.
# THIS PROGRAM REQUIRES A WORKING INSTALLATION OF SCIPY.

from numpy import *		# general NumPy stuff
from scipy import *		# general SciPy stuff
from pylab import *		# general PyLab stuff
import math				# general math stuff
import random			# random number features from Python

STEPS_PER_YEAR = 500			# set resolution here in timesteps per year
YEARS = 20						# set runtime here in number of years of fundamental orbit
NUM_STEPS = YEARS * STEPS_PER_YEAR				# total number of timesteps
PHI_LIST = linspace(0,2*pi*YEARS,NUM_STEPS)		# list of angles for the first orbit

# Ask the user: how many harmonics, "randomness" factor (intended as a positive int)
num_harmonics = int(raw_input("How many harmonics? "))
randomness = int(raw_input("How random? "))

def make_epicycle(R, phi0, omega, orbit):
	# Simulates epicycle around previously solved (orbit)
	# Inputs <float>: R = radius, phi0 = initial angle, omega = angular frequency
	# Inputs <complex list>: circle = list of points about which we will orbit
	# Output <complex list>: list of points in chronological order
	output_list = []
	for k in range(NUM_STEPS):
		# For the kth point, rotate about the origin by an angle omega * (phi + phi0).
		# Then move the center of that point to the kth point of the input circle.
		new_point = R*exp( 1j * omega * (phi0+PHI_LIST[k]) ) + orbit[k]	
		output_list.append(new_point)
	return output_list

# The first harmonic needs something around which to revolve. Here's a circle of radius 1:
circle = []
for k in range(NUM_STEPS):
	circle.append( exp( 1j * PHI_LIST[k] ) )

# Generate some random radii, initial phases, and frequencies.
# Frequencies are integer multiples of the fundamental, so orbits should be periodic.
R = []
phi0 = []
omega = []
for k in range(num_harmonics):
	R.append( 0.1 * random.uniform(1, randomness) )
	phi0.append( pi/4 * random.randrange(0,8,1) )
	omega.append( 0 )
	while omega[k] == 0 :
		omega[k] = random.randrange( -1.0 * randomness, randomness+1, 1)
	
# Calculate all orbits and put them in a list
orbits = [circle]					# 0th orbit is fundamental unit circle
for k in range(num_harmonics):
	orbits.append( make_epicycle(R[k],phi0[k],omega[k],orbits[k]) )
	print "Harmonic number", k+1, "has frequency", omega[k], "and initial phase", phi0[k]/pi, "pi."
	# Note the off-by-one issue: (orbits) alreadly included (circle) when k=0

# Turn on Pylab interactive mode so the plot window will be visible while the program runs.
ion()

# Plot the highest harmonic
plot( real( orbits[num_harmonics] ), imag( orbits[num_harmonics] ) )
# Encircle the highest harmonic with a small, non-periodic orbit.
# This makes the plot look like a 2-torus knot in 3D space.
satellite = make_epicycle(0.1, 0, 12.3456789, orbits[num_harmonics])
plot( real(satellite), imag(satellite) )

axes().set_aspect('equal')		# means "don't stretch x- and y-axes by a different amount."

quit_button = raw_input("Press Enter to quit.")		# Trick to keep the plot window open.
ioff()		# Turn off interactive mode.